"""NilmDB

Object that represents a NILM database file"""

# Need absolute_import so that "import nilmdb" won't pull in nilmdb.py,
# but will pull the nilmdb module instead.
from __future__ import absolute_import
import nilmdb
from nilmdb.printf import *

import tables
import time
import sys

class NilmDB(object):
    def __init__(self, filename):
        # Open or create the file
        self.filename = filename
        self.h5file = tables.openFile(filename, "a", "NILM Database")
        self.opened = True

    def __del__(self):
        if "opened" in self.__dict__: # pragma: no cover
            fprintf(sys.stderr,
                    "error: NilmDB.close() wasn't called, file %s",
                    self.filename)

    def close(self):
        self.h5file.close()
        del self.opened

    def stream_list(self, layout = None):
        """Return list of (path, layout) to all Tables in the
        database that match the given layout, or all Tables if
        layout is None"""
        paths = []
        for node in self.h5file.walkNodes('/', 'Table'):
            path = node._v_pathname
            this_layout = nilmdb.layout.desc_to_layout(node.description)
            if layout is None or layout == this_layout:
                paths.append((path, this_layout))
        return sorted(paths)

    def stream_create(self, path, layout_name, index = None):
        """Create a table at the given path, with the contents
        matching the given layout_name (e.g. 'PrepData').
        Columns listed in 'index' are marked as indices.  If index =
        None, the 'timestamp' column is indexed if it exists.  Pass
        an empty list to prevent indexing"""
        [ group, node ] = path.rsplit("/", 1)
        if group == '':
            raise ValueError("Invalid path")

        # Make the group structure, one element at a time
        group_path = group.lstrip('/').split("/")
        for i in range(len(group_path)):
            parent = "/" + "/".join(group_path[0:i])
            child = group_path[i]
            try:
                self.h5file.createGroup(parent, child)
            except tables.NodeError:
                pass

        # Create the table
        desc = nilmdb.layout.layout_to_desc(layout_name)
        table = self.h5file.createTable(group, node, desc)                                        

        # Create indices
        try:
            if index is None and "timestamp" in table.colnames:
                index = [ "timestamp" ]
            for ind in index:
                table.cols._f_col(ind).createIndex()
        except KeyError, e:
            # Remove this table if we got an error
            self.h5file.removeNode(group, node)
            raise e
            
